/**
 * set mode
 */
const isHtml = !!process.argv.filter(args => args === '-html').length
const isPhp = !!process.argv.filter(args => args === '-php').length


/**
 * load package
 */
import pug from 'pug'
import minify from 'html-minifier'
import { outputFile } from 'fs-extra'
import { glob } from 'glob'
import { configDotenv } from 'dotenv'
const dotenvData = configDotenv({path: '.env'})?.parsed


/**
 * 必要なURLやPATHを.envから取得
 * @requires PRODモードのパス書き換えを忘れないように
 */
let path = (() => {
  let mode = process.env.NODE_ENV ?? ''
  let strictAbsPath = [dotenvData?.DOMAIN_DEV, dotenvData?.DOCROOT_DEV, dotenvData?.SUBDIR_DEV].join('')
  let rootAbsPath = [dotenvData?.DOCROOT_DEV, dotenvData?.SUBDIR_DEV].join('')
  let innerPublicPath = [dotenvData?.PUBLIC_DEV].join('')
  let outerPublicPath = [dotenvData?.DOMAIN_DEV, dotenvData?.DOCROOT_DEV, dotenvData?.SUBDIR_DEV, dotenvData?.PUBLIC_DEV].join('')
  let version = process.env.npm_package_version ?? ''

  let isTerminate = false

  for(let i = 0, l = process.argv.length; i < l; i ++){
    switch(process.argv[i]){
      case '-local':
        mode += process.argv[i]
        strictAbsPath = [dotenvData?.DOMAIN_LOCAL, dotenvData?.DOCROOT_LOCAL, dotenvData?.SUBDIR_LOCAL].join('')
        rootAbsPath = [dotenvData?.DOCROOT_LOCAL, dotenvData?.SUBDIR_LOCAL].join('')
        innerPublicPath = [dotenvData?.PUBLIC_LOCAL].join('')
        outerPublicPath = [dotenvData?.DOMAIN_LOCAL, dotenvData?.DOCROOT_LOCAL, dotenvData?.SUBDIR_LOCAL, dotenvData?.PUBLIC_LOCAL].join('')
        isTerminate = true
      break

      case '-githubpages':
        mode += process.argv[i]
        strictAbsPath = [dotenvData?.DOMAIN_GITHUBPAGES, dotenvData?.DOCROOT_GITHUBPAGES, dotenvData?.SUBDIR_GITHUBPAGES].join('')
        rootAbsPath = [dotenvData?.DOCROOT_GITHUBPAGES, dotenvData?.SUBDIR_GITHUBPAGES].join('')
        innerPublicPath = [dotenvData?.PUBLIC_GITHUBPAGES].join('')
        outerPublicPath = [dotenvData?.DOMAIN_GITHUBPAGES, dotenvData?.DOCROOT_GITHUBPAGES, dotenvData?.SUBDIR_GITHUBPAGES, dotenvData?.PUBLIC_GITHUBPAGES].join('')
        isTerminate = true
      break

      case '-xserver':
        mode += process.argv[i]
        strictAbsPath = [dotenvData?.DOMAIN_XSERVER, dotenvData?.DOCROOT_XSERVER, dotenvData?.SUBDIR_XSERVER].join('')
        rootAbsPath = [dotenvData?.DOCROOT_XSERVER, dotenvData?.SUBDIR_XSERVER].join('')
        innerPublicPath = [dotenvData?.PUBLIC_XSERVER].join('')
        outerPublicPath = [dotenvData?.DOMAIN_XSERVER, dotenvData?.DOCROOT_XSERVER, dotenvData?.SUBDIR_XSERVER, dotenvData?.PUBLIC_XSERVER].join('')
        isTerminate = true
      break
    }

    if(isTerminate) break
  }

  return {
    mode: mode,
    strictAbsPath: strictAbsPath,
    rootAbsPath: rootAbsPath,
    innerPublicPath: innerPublicPath,
    outerPublicPath: outerPublicPath,
    version: version,
  }
})()


/**
 * pug settings
 */
const pugOptions = {
  cache: false,
  data: {
    locals: {
      mode: path.mode,
      version: path.version,
      strictAbsPath: path.strictAbsPath,
      rootAbsPath: path.rootAbsPath,
      innerPublicPath: path.innerPublicPath,
      outerPublicPath: path.outerPublicPath,
    }
  },
}


/**
 * main function
 */
(async () => {
  /**
   * collect available pug files
   */
  let files = await glob('./src/**/!(_)*.pug', {ignore: 'node_modules/**'})
  files = files.map(fname => (fname + '').replace(/\\/g, "\t").split("\t"))

  /**
   * publish files
   */
  const writeFile = (filename, htmlText) => {
    htmlText = minify.minify(htmlText, {
      ignoreCustomFragments: [/<%[\s\S]*?%>/, /<\?[\s\S]*?\?>/, /<\?php[\s\S]?\?>/],
      removeComments: true,
    })

    outputFile(filename, htmlText)
  }

  /**
   * convert pug to html/php
   */
  if(isPhp){
    files.map(farr => {
      const htmlText = pug.renderFile('./' + farr.join('/'), {
        ...pugOptions,
        filters: {
          'php': text => {
            text = text.replace(/[<]/g, '&lt;')
            text = text.replace(/[>]/g, '&gt;')
            text = '<?php ' + text + ' ?>'
            return text
          },
        },
      })
      farr[0] = './dist'
      farr[farr.length - 1] = farr[farr.length - 1].replace('.pug', '.php')
      let outstr = farr.join('/')
      writeFile(outstr, htmlText)
    })
  }else if(isHtml){
    files.map(farr => {
      const htmlText = pug.renderFile('./' + farr.join('/'), {
        ...pugOptions,
        filters: {
          'php': text => {
            text = '<!-- htmlmin:ignore --><!-- ' + text + ' --><!-- htmlmin:ignore -->'
            return text
          },
        },
      })
      farr[0] = './dist'
      farr[farr.length - 1] = farr[farr.length - 1].replace('.pug', '.html')
      let outstr = farr.join('/')
      writeFile(outstr, htmlText)
    })
  }
})()
